@@ -3,12 +3,12 @@
 #
 # Fill in `templates'
 #
-# Copyright 1996, 1997, 1999 M-J. Dominus.
+# Copyright 1996, 1997, 1999, 2001 M-J. Dominus.
 # You may copy and distribute this program under the
 # same terms as Perl iteself.  
-# If in doubt, write to mjd-perl-template@pobox.com for a license.
+# If in doubt, write to mjd-perl-template+@plover.com for a license.
 #
-# Version 1.20
+# Version 1.42
 
 package Text::Template;
 require 5.004;
@@ -18,7 +18,8 @@ use Exporter;
 use vars '$ERROR';
 use strict;
 
-$Text::Template::VERSION = '1.20';
+$Text::Template::VERSION = '1.42';
+my %GLOBAL_PREPEND = ('Text::Template' => '');
 
 sub Version {
   $Text::Template::VERSION;
@@ -30,7 +31,15 @@ sub _param {
   for $kk ($k, "\u$k", "\U$k", "-$k", "-\u$k", "-\U$k") {
     return $h{$kk} if exists $h{$kk};
   }
-  return undef;
+  return;
+}
+
+sub always_prepend
+{
+  my $pack = shift;
+  my $old = $GLOBAL_PREPEND{$pack};
+  $GLOBAL_PREPEND{$pack} = shift;
+  $old;
 }
 
 {
@@ -43,7 +52,10 @@ sub _param {
     my %a = @_;
     my $stype = uc(_param('type', %a)) || 'FILE';
     my $source = _param('source', %a);
+    my $untaint = _param('untaint', %a);
+    my $prepend = _param('prepend', %a);
     my $alt_delim = _param('delimiters', %a);
+    my $broken = _param('broken', %a);
     unless (defined $source) {
       require Carp;
       Carp::croak("Usage: $ {pack}::new(TYPE => ..., SOURCE => ...)");
@@ -53,9 +65,16 @@ sub _param {
       Carp::croak("Illegal value `$stype' for TYPE parameter");
     }
     my $self = {TYPE => $stype,
-		SOURCE => $source,
+		PREPEND => $prepend,
+                UNTAINT => $untaint,
+                BROKEN => $broken,
 		(defined $alt_delim ? (DELIM => $alt_delim) : ()),
 	       };
+    # Under 5.005_03, if any of $stype, $prepend, $untaint, or $broken
+    # are tainted, all the others become tainted too as a result of
+    # sharing the expression with them.  We install $source separately
+    # to prevent it from acquiring a spurious taint.
+    $self->{SOURCE} = $source;
 
     bless $self => $pack;
     #  $self->compile;
@@ -70,23 +89,30 @@ sub _acquire_data {
   my ($self) = @_;
   my $type = $self->{TYPE};
   if ($type eq 'STRING') {
-    return 1;
+    # nothing necessary    
   } elsif ($type eq 'FILE') {
-    my $text = _load_text($self->{SOURCE});
-    unless (defined $text) {
+    my $data = _load_text($self->{SOURCE});
+    unless (defined $data) {
       # _load_text already set $ERROR
       return undef;
     }
+    if ($self->{UNTAINT} && _is_clean($self->{SOURCE})) {
+      _unconditionally_untaint($data);
+    }
     $self->{TYPE} = 'STRING';
-    $self->{SOURCE} = $text;
+    $self->{FILENAME} = $self->{SOURCE};
+    $self->{SOURCE} = $data;
   } elsif ($type eq 'ARRAY') {
     $self->{TYPE} = 'STRING';
     $self->{SOURCE} = join '', @{$self->{SOURCE}};
   } elsif ($type eq 'FILEHANDLE') {
     $self->{TYPE} = 'STRING';
     local $/;
-    local *FH = $self->{SOURCE};
-    my $data = <FH>; # Extra assignment avoids bug in Solaris perl5.00[45].
+    my $fh = $self->{SOURCE};
+    my $data = <$fh>; # Extra assignment avoids bug in Solaris perl5.00[45].
+    if ($self->{UNTAINT}) {
+      _unconditionally_untaint($data);
+    }
     $self->{SOURCE} = $data;
   } else {
     # This should have been caught long ago, so it represents a 
@@ -94,6 +120,21 @@ sub _acquire_data {
     my $pack = ref $self;
     die "Can only acquire data for $pack objects of subtype STRING, but this is $type; aborting";
   }
+  $self->{DATA_ACQUIRED} = 1;
+}
+
+sub source {
+  my ($self) = @_;
+  $self->_acquire_data unless $self->{DATA_ACQUIRED};
+  return $self->{SOURCE};
+}
+
+sub set_source_data {
+  my ($self, $newdata) = @_;
+  $self->{SOURCE} = $newdata;
+  $self->{DATA_ACQUIRED} = 1;
+  $self->{TYPE} = 'STRING';
+  1;
 }
 
 sub compile {
@@ -180,6 +221,19 @@ sub compile {
   1;
 }
 
+sub prepend_text {
+  my ($self) = @_;
+  my $t = $self->{PREPEND};
+  unless (defined $t) {
+    $t = $GLOBAL_PREPEND{ref $self};
+    unless (defined $t) {
+      $t = $GLOBAL_PREPEND{'Text::Template'};
+    }
+  }
+  $self->{PREPEND} = $_[1] if $#_ >= 1;
+  return $t;
+}
+
 sub fill_in {
   my $fi_self = shift;
   my %fi_a = @_;
@@ -193,11 +247,19 @@ sub fill_in {
 
   my $fi_varhash = _param('hash', %fi_a);
   my $fi_package = _param('package', %fi_a) ;
-  my $fi_broken  = _param('broken', %fi_a)  || \&_default_broken;
+  my $fi_broken  = 
+    _param('broken', %fi_a)  || $fi_self->{BROKEN} || \&_default_broken;
   my $fi_broken_arg = _param('broken_arg', %fi_a) || [];
   my $fi_safe = _param('safe', %fi_a);
   my $fi_ofh = _param('output', %fi_a);
   my $fi_eval_package;
+  my $fi_scrub_package = 0;
+  my $fi_filename = _param('filename') || $fi_self->{FILENAME} || 'template';
+
+  my $fi_prepend = _param('prepend', %fi_a);
+  unless (defined $fi_prepend) {
+    $fi_prepend = $fi_self->prepend_text;
+  }
 
   if (defined $fi_safe) {
     $fi_eval_package = 'main';
@@ -205,6 +267,7 @@ sub fill_in {
     $fi_eval_package = $fi_package;
   } elsif (defined $fi_varhash) {
     $fi_eval_package = _gensym();
+    $fi_scrub_package = 1;
   } else {
     $fi_eval_package = caller;
   }
@@ -240,10 +303,13 @@ sub fill_in {
       }
     } elsif ($fi_type eq 'PROG') {
       no strict;
-      my $fi_progtext = "package $fi_eval_package; $fi_text";
+      my $fi_lcomment = "#line $fi_lineno $fi_filename";
+      my $fi_progtext = 
+        "package $fi_eval_package; $fi_prepend;\n$fi_lcomment\n$fi_text;";
       my $fi_res;
       my $fi_eval_err = '';
       if ($fi_safe) {
+        $fi_safe->reval(q{undef $OUT});
 	$fi_res = $fi_safe->reval($fi_progtext);
 	$fi_eval_err = $@;
 	my $OUT = $fi_safe->reval('$OUT');
@@ -286,13 +352,15 @@ sub fill_in {
       die "Can't happen error #2";
     }
   }
+
+  _scrubpkg($fi_eval_package) if $fi_scrub_package;
   defined $fi_ofh ? 1 : $fi_r;
 }
 
 sub fill_this_in {
   my $pack = shift;
   my $text = shift;
-  my $templ = $pack->new(TYPE => 'STRING', SOURCE => $text)
+  my $templ = $pack->new(TYPE => 'STRING', SOURCE => $text, @_)
     or return undef;
   $templ->compile or return undef;
   my $result = $templ->fill_in(@_);
@@ -300,12 +368,15 @@ sub fill_this_in {
 }
 
 sub fill_in_string {
-  Text::Template->fill_this_in(@_);
+  my $string = shift;
+  my $package = _param('package', @_);
+  push @_, 'package' => scalar(caller) unless defined $package;
+  Text::Template->fill_this_in($string, @_);
 }
 
 sub fill_in_file {
   my $fn = shift;
-  my $templ = Text::Template->new(TYPE => 'FILE', SOURCE => $fn)
+  my $templ = Text::Template->new(TYPE => 'FILE', SOURCE => $fn, @_)
     or return undef;
   $templ->compile or return undef;
   my $text = $templ->fill_in(@_);
@@ -317,8 +388,9 @@ sub _default_broken {
   my $prog_text = $a{text};
   my $err = $a{error};
   my $lineno = $a{lineno};
-  $err =~ s/\s+at .*//s;
-  "Program fragment at line $lineno delivered error ``$err''";
+  chomp $err;
+#  $err =~ s/\s+at .*//s;
+  "Program fragment delivered error ``$err''";
 }
 
 sub _load_text {
@@ -332,11 +404,31 @@ sub _load_text {
   <F>;
 }
 
+sub _is_clean {
+  my $z;
+  eval { ($z = join('', @_)), eval "#$z"; 1 }   # LOD
+}
+
+sub _unconditionally_untaint {
+  for (@_) {
+    ($_) = /(.*)/s;
+  }
+}
+
 {
   my $seqno = 0;
   sub _gensym {
     __PACKAGE__ . '::GEN' . $seqno++;
   }
+  sub _scrubpkg {
+    my $s = shift;
+    no strict 'refs';
+    my $hash = $Text::Template::{$s."::"};
+    foreach my $key (%$hash) {
+      delete $hash->{$key};
+    }
+    delete $Text::Template::{$s."::"};
+  }
 }
   
 # Given a hashful of variables (or a list of such hashes)
@@ -345,7 +437,7 @@ sub _load_text {
 sub _install_hash {
   my $hashlist = shift;
   my $dest = shift;
-  if (ref $hashlist eq 'HASH') {
+  if (UNIVERSAL::isa($hashlist, 'HASH')) {
     $hashlist = [$hashlist];
   }
   my $hash;
@@ -356,11 +448,11 @@ sub _install_hash {
       no strict 'refs';
       local *SYM = *{"$ {dest}::$name"};
       if (! defined $val) {
-	undef *{"$ {dest}::$name"};
+	delete ${"$ {dest}::"}{$name};
       } elsif (ref $val) {
 	*SYM = $val;
       } else {
-	*SYM = \$val;
+ 	*SYM = \$val;
       }
     }
   }
@@ -377,20 +469,21 @@ Text::Template - Expand template text with embedded Perl
 
 =head1 VERSION
 
-This file documents C<Text::Template> version B<1.20>
+This file documents C<Text::Template> version B<1.42>
 
 =head1 SYNOPSIS
 
  use Text::Template;
 
 
- $template = new Text::Template (TYPE => FILE,  SOURCE => 'filename.tmpl');
- $template = new Text::Template (TYPE => ARRAY, SOURCE => [ ... ] );
- $template = new Text::Template (TYPE => FILEHANDLE, SOURCE => $fh );
- $template = new Text::Template (TYPE => STRING, SOURCE => '...' );
+ $template = Text::Template->new(TYPE => FILE,  SOURCE => 'filename.tmpl');
+ $template = Text::Template->new(TYPE => ARRAY, SOURCE => [ ... ] );
+ $template = Text::Template->new(TYPE => FILEHANDLE, SOURCE => $fh );
+ $template = Text::Template->new(TYPE => STRING, SOURCE => '...' );
+ $template = Text::Template->new(PREPEND => q{use strict;}, ...);
 
  # Use a different template file syntax:
- $template = new Text::Template (DELIMITERS => [$open, $close], ...);
+ $template = Text::Template->new(DELIMITERS => [$open, $close], ...);
 
  $recipient = 'King';
  $text = $template->fill_in();  # Replaces `{$recipient}' with `King'
@@ -399,11 +492,20 @@ This file documents C<Text::Template> version B<1.20>
  $T::recipient = 'Josh';
  $text = $template->fill_in(PACKAGE => T);
 
- $hash = { recipient => 'Abed-Nego' };
- $text = $template->fill_in(HASH => $hash, ...); # Recipient is Abed-Nego
+ # Pass many variables explicitly
+ $hash = { recipient => 'Abed-Nego',
+           friends => [ 'me', 'you' ],
+           enemies => { loathsome => 'Bill Gates',
+                        fearsome => 'Larry Ellison' },
+         };
+ $text = $template->fill_in(HASH => $hash, ...);
+ # $recipient is Abed-Nego,
+ # @friends is ( 'me', 'you' ),
+ # %enemies is ( loathsome => ..., fearsome => ... )
+
 
  # Call &callback in case of programming errors in template
- $text = $template->fill_in(BROKEN => \&callback, BROKEN_ARG => [...]);
+ $text = $template->fill_in(BROKEN => \&callback, BROKEN_ARG => $ref, ...);
 
  # Evaluate program fragments in Safe compartment with restricted permissions
  $text = $template->fill_in(SAFE => $compartment, ...);
@@ -413,9 +515,11 @@ This file documents C<Text::Template> version B<1.20>
 
  # Parse template with different template file syntax:
  $text = $template->fill_in(DELIMITERS => [$open, $close], ...);
-
  # Note that this is *faster* than using the default delimiters
 
+ # Prepend specified perl code to each fragment before evaluating:
+ $text = $template->fill_in(PREPEND => q{use strict 'vars';}, ...);
+
  use Text::Template 'fill_in_string';
  $text = fill_in_string( <<EOM, PACKAGE => 'T', ...);
  Dear {$recipient},
@@ -427,6 +531,8 @@ This file documents C<Text::Template> version B<1.20>
  use Text::Template 'fill_in_file';
  $text = fill_in_file($filename, ...);
 
+ # All templates will always have `use strict vars' attached to all fragments
+ Text::Template->always_prepend(q{use strict 'vars';});
 
 =head1 DESCRIPTION
 
@@ -436,7 +542,16 @@ has little Perl programs embedded in it here and there.  When you
 `fill in' a template, you evaluate the little programs and replace
 them with their values.  
 
-Here's an example of a template:
+You can store a template in a file outside your program.  People can
+modify the template without modifying the program.  You can separate
+the formatting details from the main code, and put the formatting
+parts of the program into the template.  That prevents code bloat and
+encourages functional separation.
+
+=head2 Example
+
+Here's an example of a template, which we'll suppose is stored in the
+file C<formletter.tmpl>:
 
 	Dear {$title} {$lastname},
 
@@ -465,13 +580,31 @@ something like this:
 
 			Mark "Vizopteryx" Dominus
 
-You can store a template in a file outside your program.  People can
-modify the template without modifying the program.  You can separate
-the formatting details from the main code, and put the formatting
-parts of the program into the template.  That prevents code bloat and
-encourages functional separation.
+Here is a complete program that transforms the example
+template into the example result, and prints it out:
 
-=head1 Template Syntax
+	use Text::Template;
+
+	my $template = Text::Template->new(SOURCE => 'formletter.tmpl')
+	  or die "Couldn't construct template: $Text::Template::ERROR";
+
+	my @monthname = qw(January February March April May June
+                           July August September October November December);
+	my %vars = (title => 'Mr.',
+		    firstname => 'Bill',
+		    lastname => 'Gates',
+		    last_paid_month => 1,   # February
+		    amount => 392.12,
+		    monthname => \@monthname,
+		   );
+
+	my $result = $template->fill_in(HASH => \%vars);
+
+	if (defined $result) { print $result }
+	else { die "Couldn't fill in template: $Text::Template::ERROR" }
+
+
+=head2 Philosophy
 
 When people make a template module like this one, they almost always
 start by inventing a special syntax for substitutions.  For example,
@@ -555,7 +688,7 @@ into the last line.  The output will look something like this:
 	The Lord High Chamberlain has gotten 42
 	things for me this year.  
 
-	That is 35 more than he gave me last year.
+	That is 25 more than he gave me last year.
 
 That is all the syntax there is.  
 
@@ -619,7 +752,7 @@ that does not exist, C<$Text::Template::ERROR> will contain something like:
 This creates and returns a new template object.  C<new> returns
 C<undef> and sets C<$Text::Template::ERROR> if it can't create the
 template object.  C<SOURCE> says where the template source code will
-come from.  C<TYPE> says what kind of objec the source is.
+come from.  C<TYPE> says what kind of object the source is.
 
 The most common type of source is a file:
 
@@ -675,11 +808,44 @@ Pick a style you like and stick with it.
 =item C<DELIMITERS>
 
 You may also add a C<DELIMITERS> option.  If this option is present,
-its value should be a reference to a list of two strings.  The first
+its value should be a reference to an array of two strings.  The first
 string is the string that signals the beginning of each program
 fragment, and the second string is the string that signals the end of
 each program fragment.  See L<"Alternative Delimiters">, below.
 
+=item C<UNTAINT>
+
+If your program is running in taint mode, you may have problems if
+your templates are stored in files.  Data read from files is
+considered 'untrustworthy', and taint mode will not allow you to
+evaluate the Perl code in the file.  (It is afraid that a malicious
+person might have tampered with the file.)
+
+In some environments, however, local files are trustworthy.  You can
+tell C<Text::Template> that a certain file is trustworthy by supplying
+C<UNTAINT =E<gt> 1> in the call to C<new>.  This will tell
+C<Text::Template> to disable taint checks on template code that has
+come from a file, as long as the filename itself is considered
+trustworthy.  It will also disable taint checks on template code that
+comes from a filehandle.  When used with C<TYPE =E<gt> 'string'> or C<TYPE
+=E<gt> 'array'>, it has no effect.
+
+See L<perlsec> for more complete information about tainting.
+
+Thanks to Steve Palincsar, Gerard Vreeswijk, and Dr. Christoph Baehr
+for help with this feature.
+
+=item C<PREPEND>
+
+This option is passed along to the C<fill_in> call unless it is
+overridden in the arguments to C<fill_in>.  See L<C<PREPEND> feature
+and using C<strict> in templates> below.
+
+=item C<BROKEN>
+
+This option is passed along to the C<fill_in> call unless it is
+overridden in the arguments to C<fill_in>.  See L<C<BROKEN>> below.
+
 =back
 
 =head2 C<compile>
@@ -719,7 +885,7 @@ may yield spurious warnings about
 
 so you might like to avoid them and use the capitalized versions.
 
-At present, there are seven legal options:  C<PACKAGE>, C<BROKEN>,
+At present, there are eight legal options:  C<PACKAGE>, C<BROKEN>,
 C<BROKEN_ARG>, C<SAFE>, C<HASH>, C<OUTPUT>, and C<DELIMITERS>.
 
 =over 4
@@ -752,7 +918,8 @@ Here's an example of using C<PACKAGE>:
 	for you since 1907:
 
 	{ foreach $item (@items) {
-	    $OUT .= " * \u$item\n";
+            $item_no++;
+	    $OUT .= " $item_no. \u$item\n";
 	  }
 	}
 
@@ -767,13 +934,14 @@ C<@items>.  Here's how to do that:
 	$template->fill_in();
 
 This is not very safe.  The reason this isn't as safe is that if you
-had a variable named C<$item> in scope in your program at the point
+had a variable named C<$item_no> in scope in your program at the point
 you called C<fill_in>, its value would be clobbered by the act of
 filling out the template.  The problem is the same as if you had
-written a subroutine that used those variables in the same waythat the
-template does.  (C<$OUT> is special in templates and is always safe.)
+written a subroutine that used those variables in the same way that
+the template does.  (C<$OUT> is special in templates and is always
+safe.)
 
-One solution to this is to make the C<$item> variable private to the
+One solution to this is to make the C<$item_no> variable private to the
 template by declaring it with C<my>.  If the template does this, you
 are safe.
 
@@ -783,8 +951,8 @@ if the template does I<not> declare its variables with C<my>:
 	@Q::items = ('ivory', 'apes', 'peacocks', );
 	$template->fill_in(PACKAGE => 'Q');
 
-In this case the template will clobber the variables C<$Q::item> and
-C<$Q::list>, which are not related to the ones your program was using.
+In this case the template will clobber the variable C<$Q::item_no>,
+which is not related to the one your program was using.
 
 Templates cannot affect variables in the main program that are
 declared with C<my>, unless you give the template references to those
@@ -894,25 +1062,26 @@ fragment that cased the error.
 If you don't specify a C<BROKEN> function, C<Text::Template> supplies
 a default one that returns something like
 
-	Program fragment at line 17 delivered error ``Illegal
-	division by 0''
+	Program fragment delivered error ``Illegal division by 0 at
+	template line 37''
 
-Since this is interpolated into the template at the place the error
-occurred, a template like this one:
+(Note that the format of this message has changed slightly since
+version 1.31.)  The return value of the C<BROKEN> function is
+interpolated into the template at the place the error occurred, so
+that this template:
 
 	(3+4)*5 = { 3+4)*5 }
 
 yields this result:
 
-	(3+4)*5 = Program fragment at line 1 delivered error
-	``syntax error''
+	(3+4)*5 = Program fragment delivered error ``syntax error at template line 1''
 
 If you specify a value for the C<BROKEN> attribute, it should be a
 reference to a function that C<fill_in> can call instead of the
 default function.
 
-C<fill_in> will pass an associative array to the C<broken> function.
-The associative array will have at least these three members:
+C<fill_in> will pass a hash to the C<broken> function.
+The hash will have at least these three members:
 
 =over 4
 
@@ -922,12 +1091,16 @@ The source code of the program fragment that failed
 
 =item C<error>
 
-The text of the error message (C<$@>) generated by eval
+The text of the error message (C<$@>) generated by eval.
+
+The text has been modified to omit the trailing newline and to include
+the name of the template file (if there was one).  The line number
+counts from the beginning of the template, not from the beginning of
+the failed program fragment.
 
 =item C<lineno>
 
-The line number of the template data at which the  program fragment
-began
+The line number of the template at which the program fragment began.
 
 =back
 
@@ -1012,6 +1185,12 @@ If you use C<OUTPUT>, the return value from C<fill_in> is still true on
 success and false on failure, but the complete text is not returned to
 the caller.
 
+=item C<PREPEND>
+
+You can have some Perl code prepended automatically to the beginning
+of every program fragment.  See L<C<PREPEND> feature and using
+C<strict> in templates> below.
+
 =item C<DELIMITERS>
 
 If this option is present, its value should be a reference to a list
@@ -1047,10 +1226,10 @@ An example:
 	$Q::amount = 141.61;
 	$Q::part = 'hyoid bone';
 
-	$text = Text::Template->fill_this_in( <<EOM, PACKAGE => Q);
-	Dear {\$name},
-	You owe me \${sprintf('%.2f', \$amount)}.  
-	Pay or I will break your {\$part}.
+	$text = Text::Template->fill_this_in( <<'EOM', PACKAGE => Q);
+	Dear {$name},
+	You owe me \\${sprintf('%.2f', $amount)}.  
+	Pay or I will break your {$part}.
 		Love,
 		Grand Vizopteryx of Irkutsk.
 	EOM
@@ -1073,7 +1252,7 @@ four years ago and it is too late to change it.
 C<fill_in_string> is exactly like C<fill_this_in> except that it is
 not a method and you can omit the C<Text::Template-E<gt>> and just say
 
-	print fill_in_string(<<EOM, ...);
+	print fill_in_string(<<'EOM', ...);
 	Dear {$name},
 	  ...
 	EOM
@@ -1157,10 +1336,17 @@ scope in which they're declared.  The template is not part of that
 scope, so the template can't see C<$recipient>.  
 
 If that's not the behavior you want, don't use C<my>.  C<my> means a
-proivate variable, and in this case you don't want the variable to be
+private variable, and in this case you don't want the variable to be
 private.  Put the variables into package variables in some other
-package, and use the C<PACKAGE> option to C<fill_in>, or pass the
-names and values in a hash with the C<HASH> option.
+package, and use the C<PACKAGE> option to C<fill_in>:
+
+	$Q::recipient = $recipient;
+	my $text = fill_in_file('formletter.tmpl', PACKAGE => 'Q');
+	
+
+or pass the names and values in a hash with the C<HASH> option:
+
+	my $text = fill_in_file('formletter.tmpl', HASH => { recipient => $recipient });
 
 =head2 Security Matters
 
@@ -1197,7 +1383,8 @@ variables all have names that begin with C<$fi_>, so if you stay away
 from those names you'll be safe.  (Of course, if you're a real wizard
 you can tamper with them deliberately for exciting effects; this is
 actually how C<$OUT> works.)  I can fix this, but it will make the
-package slower to do it, so I would prefer not to.
+package slower to do it, so I would prefer not to.  If you are worried
+about this, send me mail and I will show you what to do about it.
 
 =head2 Alternative Delimiters
 
@@ -1212,7 +1399,7 @@ an alternative set of delimiters with the C<DELIMITERS> option.  For
 example, if you would like code fragments to be delimited by C<[@-->
 and C<--@]> instead of C<{> and C<}>, use
 
-	... DELIMITERS =E<gt> [ '[@--', '--@]' ], ...
+	... DELIMITERS => [ '[@--', '--@]' ], ...
 
 Note that these delimiters are I<literal strings>, not regexes.  (I
 tried for regexes, but it complicates the lexical analysis too much.)
@@ -1221,7 +1408,7 @@ backslash, so if you want to include the delimiters in the literal
 text of your template file, you are out of luck---it is up to you to
 choose delimiters that do not conflict with what you are doing.  The
 delimiter strings may still appear inside of program fragments as long
-as they nest properly.  This means that if sor sume reason you
+as they nest properly.  This means that if for some reason you
 absolutely must have a program fragment that mentions one of the
 delimiters, like this:
 
@@ -1245,6 +1432,119 @@ parsing process by 20-25%.  This shows that my original choice of C<{>
 and C<}> was very bad.  I therefore recommend that you use alternative
 delimiters whenever possible. 
 
+=head2 C<PREPEND> feature and using C<strict> in templates
+
+Suppose you would like to use C<strict> in your templates to detect
+undeclared variables and the like.  But each code fragment is a
+separate lexical scope, so you have to turn on C<strict> at the top of
+each and every code fragment:
+
+	{ use strict;
+	  use vars '$foo';
+	  $foo = 14;
+	  ...
+	}
+
+	...
+
+	{ # we forgot to put `use strict' here
+	  my $result = $boo + 12;    # $boo is misspelled and should be $foo
+	  # No error is raised on `$boo'
+	}
+
+Because we didn't put C<use strict> at the top of the second fragment,
+it was only active in the first fragment, and we didn't get any
+C<strict> checking in the second fragment.  Then we mispelled C<$foo>
+and the error wasn't caught.  
+
+C<Text::Template> version 1.22 and higher has a new feature to make
+this easier.  You can specify that any text at all be automatically
+added to the beginning of each program fragment.  
+
+When you make a call to C<fill_in>, you can specify a
+
+	PREPEND => 'some perl statements here'
+
+option; the statements will be prepended to each program fragment for
+that one call only.  Suppose that the C<fill_in> call included a
+
+	PREPEND => 'use strict;'
+
+option, and that the template looked like this:
+
+	{ use vars '$foo';
+	  $foo = 14;
+	  ...
+	}
+
+	...
+
+	{ my $result = $boo + 12;    # $boo is misspelled and should be $foo
+	  ...
+	}
+
+The code in the second fragment would fail, because C<$boo> has not
+been declared.  C<use strict> was implied, even though you did not
+write it explicitly, because the C<PREPEND> option added it for you
+automatically.
+
+There are two other ways to do this.  At the time you create the
+template object with C<new>, you can also supply a C<PREPEND> option,
+in which case the statements will be prepended each time you fill in
+that template.  If the C<fill_in> call has its own C<PREPEND> option,
+this overrides the one specified at the time you created the
+template.  Finally, you can make the class method call
+
+	Text::Template->always_prepend('perl statements');
+
+If you do this, then call calls to C<fill_in> for I<any> template will
+attach the perl statements to the beginning of each program fragment,
+except where overridden by C<PREPEND> options to C<new> or C<fill_in>.
+
+=head2 Prepending in Derived Classes
+
+This section is technical, and you should skip it on the first few
+readings. 
+
+Normally there are three places that prepended text could come from.
+It could come from the C<PREPEND> option in the C<fill_in> call, from
+the C<PREPEND> option in the C<new> call that created the template
+object, or from the argument of the C<always_prepend> call.
+C<Text::Template> looks for these three things in order and takes the
+first one that it finds.
+
+In a subclass of C<Text::Template>, this last possibility is
+ambiguous.  Suppose C<S> is a subclass of C<Text::Template>.  Should 
+
+	Text::Template->always_prepend(...);
+
+affect objects in class C<Derived>?  The answer is that you can have it
+either way.  
+
+The C<always_prepend> value for C<Text::Template> is normally stored
+in  a hash variable named C<%GLOBAL_PREPEND> under the key
+C<Text::Template>.  When C<Text::Template> looks to see what text to
+prepend, it first looks in the template object itself, and if not, it
+looks in C<$GLOBAL_PREPEND{I<class>}> where I<class> is the class to
+which the template object belongs.  If it doesn't find any value, it
+looks in C<$GLOBAL_PREPEND{'Text::Template'}>.  This means that
+objects in class C<Derived> I<will> be affected by
+
+	Text::Template->always_prepend(...);
+
+I<unless> there is also a call to
+
+	Derived->always_prepend(...);
+
+So when you're designing your derived class, you can arrange to have
+your objects ignore C<Text::Template::always_prepend> calls by simply
+putting C<Derived-E<gt>always_prepend('')> at the top of your module.
+
+Of course, there is also a final escape hatch: Templates support a
+C<prepend_text> that is used to look up the appropriate text to be
+prepended at C<fill_in> time.  Your derived class can override this
+method to get an arbitrary effect.
+
 =head2 JavaScript
 
 Jennifer D. St Clair asks:
@@ -1296,10 +1596,9 @@ it would turn into
 
 So for your JavaScript, just write
 
-	    {q{
-	      if (br== "n3") { 
-	  	  // etc.
-	      }
+	    {q{if (br== "n3") { 
+	  	 // etc.
+	       }}
 	    }
 
 and it'll come out as
@@ -1337,31 +1636,35 @@ To prevent the 17 from appearing in the output is very simple:
 	}
 
 Now the last expression evaluated yields the empty string, which is
-invisible.
+invisible.  If you don't like the way this looks, use
+
+	{ ...
+	  $var = 17;
+	  ($SILENTLY);
+	}
+
+instead.  Presumably, C<$SILENTLY> has no value, so nothing will be
+interpolated.  This is what is known as a `trick'.
 
 =head2 Compatibility
 
 Every effort has been made to make this module compatible with older
-versions.  There are three exceptions.  One is the output format of
-the default C<BROKEN> subroutine; I decided that the old format was
-too verbose.  If this bothers you, it's easy to supply a custom
-subroutine that yields the old behavior.  The second is that the
-C<$OUT> feature arrogates the C<$OUT> variable for itself.  If you had
-templates that happened to use a variable named C<$OUT>, you will have
-to change them to use some other variable or all sorts of strangeness
-may result.
+versions.  The only known exceptions follow:
 
-The third incompatibility is with the behavior of the \ metacharacter.
-In 0.1b, \\ was special everywhere, and the template processor always
-replaced it with a single backslash before passing the code to Perl
-for evaluation.  The rule now is more complicated but probably more
-convenient.  See the section on backslash processing, below, for a
-full discussion.
+The output format of the default C<BROKEN> subroutine has changed
+twice, most recently between versions 1.31 and 1.40.
 
-With a minor change to fix the format of the default C<BROKEN>
-subroutine, this version passes the test suite from the old version.
-(It is in C<t/01-basic.t>.) The old test suite was too small, but it's
-a little reassuring.
+Starting in version 1.10, the C<$OUT> variable is arrogated for a
+special meaning.  If you had templates before version 1.10 that
+happened to use a variable named C<$OUT>, you will have to change them
+to use some other variable or all sorts of strangeness will result.
+
+Between versions 0.1b and 1.00 the behavior of the \ metacharacter
+changed.  In 0.1b, \\ was special everywhere, and the template
+processor always replaced it with a single backslash before passing
+the code to Perl for evaluation.  The rule now is more complicated but
+probably more convenient.  See the section on backslash processing,
+below, for a full discussion.
 
 =head2 Backslash Processing
 
@@ -1372,9 +1675,9 @@ generate a backslash, because the code passed to Perl for evaluation
 was C<"\"> which is a syntax error.  If you wanted a backslash, you
 would have had to write C<{"\\\\"}>.
 
-In C<Text::Template> versions 1.0 through 1.10, there was a bug:
+In C<Text::Template> versions 1.00 through 1.10, there was a bug:
 Backslash was special everywhere.  In these versions, C<{"\n"}>
-generated the letter C<n>.  
+generated the letter C<n>.
 
 The bug has been corrected in version 1.11, but I did not go back to
 exactly the old rule, because I did not like the idea of having to
@@ -1463,12 +1766,17 @@ inside the template:
 			    );
 	}
 
+=head2 Automatic preprocessing of program fragments
+
+It may be useful to preprocess the program fragments before they are
+evaluated.  See C<Text::Template::Preprocess> for more details.
 
 =head2 Author
 
 Mark-Jason Dominus, Plover Systems
 
-C<mjd-perl-template@pobox.com>
+Please send questions and other remarks about this software to
+C<mjd-perl-template+@plover.com>
 
 You can join a very low-volume (E<lt>10 messages per year) mailing
 list for announcements about this package.  Send an empty note to
@@ -1478,36 +1786,70 @@ For updates, visit C<http://www.plover.com/~mjd/perl/Template/>.
 
 =head2 Support?
 
-This software is version 1.0.  It is a complete rewrite of an older
-package, and may have bugs.  Suggestions and bug reports are always
-welcome.  Send them to C<mjd-perl-template@plover.com>.  (That is my
-address, not the address of the mailing list.  The mailing list
-address is a secret.)
+This software is version 1.42.  It may have bugs.  Suggestions and bug
+reports are always welcome.  Send them to
+C<mjd-perl-template+@plover.com>.  (That is my address, not the address
+of the mailing list.  The mailing list address is a secret.)
 
-=head2 Thanks
+=head1 LICENSE
+
+    Text::Template version 1.42
+    Copyright (C) 2001 Mark Jason Dominus
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License as
+    published by the Free Software Foundation; either version 2 of the
+    License, or (at your option) any later version.  You may also can
+    redistribute it and/or modify it under the terms of the Perl
+    Artistic License.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received copies of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+
+=head1 THANKS
 
 Many thanks to the following people for offering support,
-encouragement, advice, and all the other good stuff.  
+encouragement, advice, bug reports, and all the other good stuff.  
 
+David H. Adler /
+Joel Appelbaum /
 Klaus Arnhold /
+Kevin Atteson /
+Chris.Brezil /
 Mike Brodhead /
 Tom Brown /
 Tim Bunce /
 Juan E. Camacho /
+Itamar Almeida de Carvalho /
 Joseph Cheek /
+Gene Damon /
 San Deng /
 Bob Dougherty /
 Dan Franklin /
+gary at dls.net /
 Todd A. Green /
+Donald L. Greer Jr. /
 Michelangelo Grigni /
 Tom Henry /
 Matt X. Hunter /
 Robert M. Ioffe /
 Daniel LaLiberte /
 Reuven M. Lerner /
+Trip Lilley / 
+Yannis Livassof /
+David Marshall /
 Joel Meulenberg /
 Jason Moore /
+Chris Nandor /
 Bek Oberin /
+Steve Palincsar /
 Ron Pero /
 Hans Persson /
 Jonathan Roy /
@@ -1527,8 +1869,10 @@ Shad Todd /
 Lorenzo Valdettaro /
 Larry Virden /
 Andy Wardley /
+Archie Warnock /
 Matt Womer /
 Andrew G Wood /
+Daini Xie /
 Michaely Yeung
 
 Special thanks to:
@@ -1547,12 +1891,6 @@ helping me figure out the Right Thing, and, especially, for talking me
 out of adding any new syntax.  These discussions resulted in the
 C<$OUT> feature.
 
-=item Shabbir J. Safdar
-
-for volunteering to maintain a slightly modified version of
-C<Text::Template> which will run under perl.5003 and other early
-versions. 
-
 =back
 
 =head2 Bugs and Caveats
@@ -1565,13 +1903,9 @@ The line number information will be wrong if the template's lines are
 not terminated by C<"\n">.  You should let me know if this is a
 problem.  If you do, I will fix it.
 
-The default format for reporting of broken program fragments has
-changed since version 0.1.  
-
 The C<$OUT> variable has a special meaning in templates, so you cannot
 use it as if it were a regular variable.
 
 There are not quite enough tests in the test suite.
 
 =cut
-
